/* Chromium Unity integration extension
 * 
 *   Copyright 2012 Canonical Ltd.
 *
 *   This program is free software: you can redistribute it and/or modify it 
 *   under the terms of the GNU General Public License version 3, as published 
 *   by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranties of 
 *   MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 *   PURPOSE.  See the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along 
 *   with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

(function () {
   
   // TODO extract based on logging settings
   var log = {
     info: function (msg) {
       if (msg) {
	 console.log ('Info: ' + msg);
       }
     }
     ,
     error: function (msg) {
       if (msg) {
	 console.log ('Error: ' + msg);
       }
     }
   };
   
   var plugin = document.getElementById ('unityChromiumExtensionId');
   if ( ! plugin) {
     log.error ("Unable to retrieve the Unity Chromium plugin");
     return;
   }
   var service = plugin.service_new ();
   if (! service) {
     log.error ("Unable to retrieve a connection to Unity Webapps service");
     return;
   }

   var loginListeners = [];

   var onLoginListenerRequest = function (request, sender, sendResponse) {
     if (request.action == "registerLoginListener") {
       log.info ("Login listener request from " + sender.id);
       loginListeners.push (sender.id);
     }
   };
   chrome.extension.onRequestExternal.addListener (onLoginListenerRequest);

   var onLoginEvent = function (request, sender, sendResponse) {
     if (request.action == "loginEvent") {
       log.info ("Login on " + request.domain + ": " + request.login);
       var len = loginListeners.length;
       for (var i = 0; i < len; i++) {
         var extensionId = loginListeners[i];
         log.info("Notifying login listener: " + extensionId);
         chrome.extension.sendRequest (extensionId, {
                                         type: "login",
                                         login: request.login,
                                         domain: request.domain
                                       });
       }
     }
   };
   chrome.extension.onRequest.addListener (onLoginEvent);

   var gatherUserScriptsDescriptions = function (plugin) {
     
     var repo = plugin.get_integration_scripts_repo ();
     return repo.getIntegrationScripts ();
   };

   var buildIntegrationScriptDB = function (plugin) {
     var db = [];
     
     var integrationScripts = null;
     try {
       integrationScripts = gatherUserScriptsDescriptions (plugin);
     }
     catch (e) {
       log.error ("Error while trying to gather user script descriptions: " + String(e));
       return db;
     }
     
     if ( ! integrationScripts) {
       return db;
     }
     
     log.info ("Found " + integrationScripts.length + " integration scripts");
     
     for (var i = 0; i < integrationScripts.length; ++i) {
       
       if ( ! integrationScripts[i]) {
	 log.error ('Error while going through the integration scripts: null for index ' + i);
	 continue;
       }
       
       var script = {};
       
       script.includes = integrationScripts[i].includes.split (';');
       script.includes = script.includes.map (
	 function (incl) {
	   var p = null;
	   if (incl && incl.length !== 0) {
	     try { p = new MatchPattern (incl); } catch (e) {};
	   }
	   return p;
	 }
       );
       script.includes = script.includes.filter (function (elt) { return null !== elt; });
       
       script.name = integrationScripts[i].name + "";
       script.content = integrationScripts[i].content + "";
       script.requires = integrationScripts[i].requires + "";
       
       // TODO: mmmh keep a ref?
       script.ref = integrationScripts[i];
       
       db.push (script);
     }
     
     return db;
   };
   var db = buildIntegrationScriptDB (plugin);
   
   
   // performs a match on the list of loaded integration scripts
   // given a url.
   var matchesIntegrationScript = function (url, scripts) {
     function testMatch(matchPattern) {
       return matchPattern.doMatch(url);
     };
     
     for (var i = 0; i < scripts.length; ++i) {
       var script = scripts[i];
       try {
	 if (script.includes && script.includes.some (testMatch)) {
	   return script;
	 }
       }
       catch (err)
       {}
     }
     return null;
   };

   // handles tab change & activation code
   chrome.tabs.onActivated.addListener (
     function (info) {
       // TODO: pretty bad ... assumed that all tabs are interested
       chrome.windows.getAll ({populate: true}
			      , function (windows) {
				for (var winIdx in windows) {
				  var tabs = windows[winIdx].tabs;
				  for (var tabIdx in tabs) {
				    var tabId = tabs[tabIdx].id;
				    chrome.tabs.sendRequest (tabId
							     , {method: "on_tab_active_changed"
								, tabId: info.tabId
							       }
							     , function (response) {});
				  } // for (var tabIdx
				} // for (var winIdx
			      });
     }
   );

   // TODO move elsewhere
   // TODO check that's it's fine (it should), since contrary to
   //      FF this is being called on the same environment every time (background page)
   function toDataURL (uri, callback, sx, sy, sw, sh) {
     if (uri.match(/^\s{0,}file:/i))
       return;
     
     if (uri.match(/^\s{0,}https:/i))
       return;
     
     var self = this;
     var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
     var img = new window.Image();
     
     img.onload = function() {
       var width = img.width, height = img.height;

       var hasExtraArgs = sx || sx === 0;
       if (hasExtraArgs) {
	 width = sw;
	 height = sh;
       }

       var dx  = 0, dy = 0;

       if (width > height)
	 dy = width - height;
       else
	 dx = height - width;

       canvas.width = width + dx;
       canvas.height = height + dy;
       
       var ctx = canvas.getContext('2d');
       if (hasExtraArgs)
	 ctx.drawImage(img, sx, sy, sw, sh, dx / 2, dy / 2, width, height);
       else
	 ctx.drawImage(img, dx / 2, dy / 2);
       
       callback (true, canvas.toDataURL());
     };
     img.src = uri;
   };
   
   
   ////////////////////////////////////////
   // 
   // main request handler
   // 
   ///////////////////////////////////////
   
   // list of callback that are to be called asynchronously
   //  per tab. Used in the user integration resquests contex.
   var user_integration_callbacks = {};
   
   var contentScriptsRequestHandler = function (request, sender, callback) {
     
     var addIntegrationCallbackFor = function (integrationId, callback, message, details) {
       
       user_integration_callbacks[integrationId] = {callback: callback
						    , message: message
                                                    , details: details
						   };
     };
     var getDataIfAnyFor = function (integrationId) {
       
       if (user_integration_callbacks[integrationId] === undefined) {
	 return "";
       }
       return { message: user_integration_callbacks[integrationId].message,
                details: user_integration_callbacks[integrationId].details };
     };
     var invokeAndRemoveCallbackIfAnyFor = function (integrationId, arguments) {
       
       if (user_integration_callbacks[integrationId] === undefined) {
	 return;
       }
       var callback = user_integration_callbacks[integrationId].callback;
       user_integration_callbacks[integrationId] = undefined;
       if (typeof(callback) === 'function') {
	 callback(arguments);
       }
     };

     var handlers = {
       get_unity_script: function (request, sender, callback) {
	 
	 var result = {
	   userscript: null
	   , requires: null
	 };
	 var script = matchesIntegrationScript (request.url, db);
	 if (script) {
	   result.userscript = script.content;
	   if (script.requires && script.requires.length !== 0) {
	     result.requires = script.requires;
	   }
	 }
	 callback (result);
       }
       ,
       get_base_domain: function (request, sender, callback) {
	 
	 chrome.tld.getBaseDomain (request.url, callback);
       }
       , 
       get_message_for_user_integration: function (request, sender, callback) {

	 callback (getDataIfAnyFor (request.tabId));
       }
       ,
       add_integrationscript_for_tabid: function (request, sender, callback) {
         
         addIntegrationScriptForTab(sender.tab.id, request.name, request.domain, request.interest_id);
       }
       ,
       on_user_integration_result: function (request, sender, callback) {
	 
	 invokeAndRemoveCallbackIfAnyFor (request.tabId, request);
       }
       ,
       request_user_integration: function (request, sender, callback) {
	 
	 addIntegrationCallbackFor (sender.tab.id, callback, request.message, request.details);
	 
	 chrome.infobars.show ({tabId: sender.tab.id, path: "infobar.html"});
       }
       , 
       take_snapshot: function (request, sender, callback) {
	 
	 chrome.tabs.captureVisibleTab (sender.tab.windowId
					, null
					, function (dataURL) {
					  callback (dataURL);
					});
       }
       , 
       get_data_url_from: function (request, sender, callback) {
	 
	 toDataURL (request.url
		    , function (flag, dataUrl) {
		      
		      callback ({flag: flag, dataUrl: dataUrl});
		    });
       }
       , 
       get_extension_settings: function (request, sender, callback) {
	 
	 var settings = {
	   logging: false,
           incognito: sender.tab.incognito
	 };
	 
	 try {
	   if (window.localStorage) {
	     settings.logging = localStorage['logging'];
	   }
	 }
	 catch (e) {
	   log.error ("Error while trying to retrieve logging information: " + str(e));
	 }
	 
	 callback (settings);
       }
       , 
       get_tab_info: function (request, sender, callback) {
	 
	 callback ({tabId: sender.tab.id, windowId: sender.tab.windowId});
       }
       , 
       kill_this_tab: function (request, sender, callback) {
	 
	 chrome.tabs.remove ([request.tabId], function () {});
       }
       , 
       raise_this_tab: function (request, sender, callback) {
	 
	 // bring the window & the tab to the front
	 chrome.windows.update (request.windowId
				, {focused: true, state: "maximized"}
				, function (window) {
				  chrome.tabs.update (request.tabId
						      , {active: true, highlighted: true});
				});
       }
       , 
       is_selected: function (request, sender, callback) {
	 
	 chrome.tabs.getCurrent (
	   function (tab) {
	     if (tab) {
	       callback ({isSelected: tab.id === request.tabId});
	     }
	   }
	 );
       }
       
     }; // handlers
     
     
     // validate request
     if ( ! request  || ! request.method) {
       callback ({error: "Invalid request structure"});
       return;
     }
     if ( ! sender) {
       callback ({error: "Invalid sender"});
       return;
     }
     if ( typeof (request.method) != 'string' || request.method.length == 0) {
       callback ({error: "Invalid request method"});
       return;
     }
     
     log.info ('Got request: ' + request.method + ', ' + sender.tab.id);
     
     var handler = handlers [request.method];
     
     if (handler !== undefined && typeof(handler) == 'function') {
       
       log.info ('Calling handler for: ' + request.method);
       
       handler (request, sender, callback);
     }
   };
   
   // main event handler
   chrome.extension.onRequest.addListener (contentScriptsRequestHandler);
   
   // tab management handling
   var integration_script_tabs = {};
   var addIntegrationScriptForTab = function (tabId, name, domain, interest_id) {
     if (integration_script_tabs[tabId] !== undefined) {
       log.info('Overriding tab information for existing tab: ' + tabId);
     }
     log.info ('Integrating tab with name: ' + name);
     integration_script_tabs[tabId] = {name: name, domain: domain, interest_id: interest_id};
   };
   var removeIntegrationScriptForTab = function (tabId) {
     if (integration_script_tabs[tabId] !== undefined) {
       integration_script_tabs[tabId] = undefined;
     }
   };
   chrome.tabs.onRemoved.addListener (
     function (tabId, removeInfo) {
       var info = integration_script_tabs[tabId];
       removeIntegrationScriptForTab (tabId);
       if (info !== undefined) {
         log.info('Destroying interest: ' + info.name + ', ' + info.domain + ', ' + info.interest_id);
         try {
           plugin.service_destroy_interest_for_context (service
                                                        , info.name
                                                        , info.domain
                                                        , info.interest_id
                                                        , 1);
         }
         catch (e) {
           log.error('Error while trying to destroy interest for tab ' + tabId + ', ' + e);
         }
       }
     }
   );

   // misc (to remove)
   chrome.tabs.onCreated.addListener (
     function (tab) {
       var script = matchesIntegrationScript (tab.url, db);
       if (script) {
	 log.info ("New tab created for URL: " + tab.url + ", matches an integration script: " + script);
       }
     }
   );

 }) ();

