/*
 * unity-webapps-repo-userscript-wrapper.c
 * Copyright (C) Canonical LTD 2012
 * 
 * Author: Alexandre Abreu <alexandre.abreu@canonical.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include <memory.h> // TODO remove

#include <glib.h>

#include <libunity_webapps-0.2/unity-webapps-script-repo.h>

#include "npapi-headers/headers/npfunctions.h"
#include "npapi-headers/headers/nptypes.h"
#include "npapi-headers/headers/npruntime.h"

#include "unity-webapps-repo-userscript-wrapper.h"
#include "unity-npapi-common.h"
#include "unity-npapi-debug.h"
#include "unity-webapps-base-js-object.h"


static NPVariant
get_integration_scripts (NPP instance
			 , NPObject * object
			 , const NPVariant args[]
			 , uint32_t argcnt);

static NPVariant
get_integration_script_by_idx (NPP instance
			       , NPObject * object
			       , int32_t idx);

static NPVariant
integration_script_count_property_get (NPObject * object);




///////////////////////////////////////////////////////
//
//	public
//
///////////////////////////////////////////////////////


NPObject *
unity_webapps_npapi_create_integrationscript_repo_wrapper_for (NPP instance)
{
  NPAPI_LOG (INFO, "%s", __func__);
  
  NPObject *
    pObject = unity_webapps_npapi_create_js_object (instance, "Integration Script Repository Wrapper");
  
  g_return_val_if_fail (NULL != pObject, NULL);
  
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = (UnityWebappsJavascriptObjectWrapper *) pObject;
  
  unity_webapps_npapi_add_method (pWrapper, "getIntegrationScripts", get_integration_scripts);
  
  return pObject;
}



///////////////////////////////////////////////////////
//
//	private
//
///////////////////////////////////////////////////////


static NPVariant
integration_script_name_property_get (NPObject * object)
{
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = (UnityWebappsJavascriptObjectWrapper *) object;
  
  NPVariant result;
  NULL_TO_NPVARIANT(result);
  
  g_return_val_if_fail (NULL != pWrapper, result);
  
  UnityWebappsIntegrationScriptDescription *
    pDescription = (UnityWebappsIntegrationScriptDescription *) pWrapper->reserved;
  
  g_return_val_if_fail (NULL != pDescription, result);
  
  gint size = strlen(pDescription->name) + 1;
  gchar * ret = NPN_MemAlloc (size);
  
  g_return_val_if_fail (NULL != ret, result);
  
  memset (ret, 0, size);
  g_strlcpy (ret, pDescription->name, size);
  
  STRINGZ_TO_NPVARIANT(ret, result);
  
  return result;
}


static NPVariant
integration_script_includes_property_get (NPObject * object)
{
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = (UnityWebappsJavascriptObjectWrapper *) object;
  
  NPVariant result;
  NULL_TO_NPVARIANT(result);
  
  g_return_val_if_fail (NULL != pWrapper, result);
  
  UnityWebappsIntegrationScriptDescription *
    pDescription = (UnityWebappsIntegrationScriptDescription *) pWrapper->reserved;
  
  g_return_val_if_fail (NULL != pDescription, result);
  g_return_val_if_fail (NULL != pDescription->includes, result);  
  
  // actually get the includes
  
  gsize count = 0;
  GSList * i = pDescription->includes;
  for (; i != NULL; i = g_slist_next (i))
    {
      if (NULL != i && NULL != i->data)
	{
	  count += (NULL != i->data) ? (strlen(i->data) + 1) : 0;
	  count += 1; // for separator
	}
    }
  
  g_return_val_if_fail (count > 0, result);
    
  gchar * ret = NPN_MemAlloc (count);
  g_return_val_if_fail (NULL != ret, result);
  memset (ret, 0, count);
  
  const char * SEPARATOR = ";";
  
  i = pDescription->includes;
  for (; i != NULL; i = g_slist_next (i))
    {
      if (NULL != i && i->data)
	{
	  g_strlcat (ret, i->data, count);
	  g_strlcat (ret, SEPARATOR, count);
	}
    }
  
  STRINGZ_TO_NPVARIANT(ret, result);
  
  return result;
}


static NPVariant
integration_script_content_property_get (NPObject * object)
{
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = (UnityWebappsJavascriptObjectWrapper *) object;
  
  NPVariant result;
  NULL_TO_NPVARIANT(result);
  
  g_return_val_if_fail (NULL != pWrapper, result);
  
  UnityWebappsIntegrationScriptDescription *
    pDescription = (UnityWebappsIntegrationScriptDescription *) pWrapper->reserved;
  
  g_return_val_if_fail (NULL != pDescription, result);
  
  gint size = strlen(pDescription->content) + 1;
  gchar * ret = NPN_MemAlloc (size);
  g_return_val_if_fail (NULL != ret, result);
  
  memset (ret, 0, size);
  g_strlcpy (ret, pDescription->content, size);
  
  STRINGZ_TO_NPVARIANT(ret, result);
  
  return result;
}


static NPVariant
integration_script_requires_property_get (NPObject * object)
{
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = (UnityWebappsJavascriptObjectWrapper *) object;
  
  NPVariant result;
  NULL_TO_NPVARIANT(result);
  
  g_return_val_if_fail (NULL != pWrapper, result);
  
  UnityWebappsIntegrationScriptDescription *
    pDescription = (UnityWebappsIntegrationScriptDescription *) pWrapper->reserved;
  
  g_return_val_if_fail (NULL != pDescription, result);
  
  if (NULL == pDescription->requires)
    {
      return result;
    }
  
  GSList *
    values = pDescription->requires;
  g_return_val_if_fail (NULL != values, result);
  
  // does the work, simply returns all the required files together
  const gchar * SEPARATOR = "\n\n";
  
  gsize count = 0;
  GSList * i = values;
  for (; i != NULL; i = g_slist_next (i))
    {
      if (NULL != i && NULL != i->data)
	{
	  UnityWebappsIntegrationScriptContent
	    * pContent = (UnityWebappsIntegrationScriptContent *) i->data;
	  
	  count += (NULL != pContent->content) ? (strlen(pContent->content) + 1) : 0;
	  count += strlen(SEPARATOR);
	}
    }
  
  g_return_val_if_fail (count > 0, result);
  
  gchar * ret = NPN_MemAlloc (count);
  g_return_val_if_fail (NULL != ret, result);
  memset (ret, 0, count);
  
  i = values;
  for (; i != NULL; i = g_slist_next (i))
    {
      if (NULL != i && i->data)
	{
	  UnityWebappsIntegrationScriptContent
	    * pContent = (UnityWebappsIntegrationScriptContent *) i->data;
	  
	  g_strlcat (ret, pContent->content, count);
	  g_strlcat (ret, SEPARATOR, count);
	}
    }
  
  STRINGZ_TO_NPVARIANT(ret, result);
  
  return result;
}



static NPObject *
create_integration_script_object_for (NPP instance
				      , UnityWebappsIntegrationScriptDescription * pDescription)
{
  g_return_val_if_fail (NULL != pDescription, NULL);
  
  NPObject *
    pObject = unity_webapps_npapi_create_js_object (instance, "Integration Script Wrapper");
  
  g_return_val_if_fail (NULL != pObject, NULL);
  
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = ((UnityWebappsJavascriptObjectWrapper *) pObject);
  
  unity_webapps_npapi_add_property (pWrapper, "name", integration_script_name_property_get, NULL);
  unity_webapps_npapi_add_property (pWrapper, "includes", integration_script_includes_property_get, NULL);
  unity_webapps_npapi_add_property (pWrapper, "content", integration_script_content_property_get, NULL);
  unity_webapps_npapi_add_property (pWrapper, "requires", integration_script_requires_property_get, NULL);
  
  // TODO watch out !!! this is a reference to something that could be (potentially) GCed by the browser if
  //  the repository structure is not being references anymore ... make copy (or incr-ref)
  pWrapper->reserved = pDescription;
  
  return pObject;
}


static NPVariant
get_integration_script_by_idx (NPP instance
			       , NPObject * object
			       , int32_t idx)
{
  NPAPI_LOG (INFO, "%s", __func__);
  
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = (UnityWebappsJavascriptObjectWrapper *) object;
  
  NPVariant result;
  NULL_TO_NPVARIANT (result);
  
  g_return_val_if_fail (NULL != pWrapper, result);
  g_return_val_if_fail (NULL != pWrapper->reserved, result);
  g_return_val_if_fail (idx >= 0, result);
  
  gsize count = 0;
  
  UnityWebappsIntegrationScriptDescription **
    ppDescription = pWrapper->reserved;
  while (ppDescription[count] != NULL) ++count;
  
  g_return_val_if_fail (idx < count, result);
  
  g_return_val_if_fail (NULL != ppDescription[idx], result);  
  
  NPObject *
    pObject = create_integration_script_object_for (pWrapper->instance, ppDescription[idx]);
  
  g_return_val_if_fail (NULL != pObject, result);
  
  OBJECT_TO_NPVARIANT(pObject, result);
  
  return result;
}


static void
free_repo_descriptions (NPP instance, NPObject * object)
{
  NPAPI_LOG (INFO, "%s", __func__);
  
  g_return_if_fail (NULL != object);
  
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = ((UnityWebappsJavascriptObjectWrapper *) object);
  
  if (NULL == pWrapper->reserved)
    {
      return;
    }
  
  unity_webapps_script_repo_free_all (pWrapper->reserved);
  
  pWrapper->reserved = NULL;
}


static NPObject *
create_userscript_repo_result_object (NPP instance)
{
  NPAPI_LOG (INFO, "%s", __func__);
  
  // init
  UnityWebappsIntegrationScriptDescription **
    ppDescription = unity_webapps_script_repo_get_descriptions ();
  
  if (NULL == ppDescription)
    {
      g_critical ("Could not retrieve Unity Webapps repository");
      return NULL;
    }
  
  NPObject *
    pObject = unity_webapps_npapi_create_js_object (instance, "Integration Scripts Results Wrapper");
  
  if (NULL == pObject)
    {
      unity_webapps_script_repo_free_all (ppDescription);
      return NULL;
    }
  
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = ((UnityWebappsJavascriptObjectWrapper *) pObject);
  
  // setup object
  unity_webapps_npapi_add_property (pWrapper, "length", integration_script_count_property_get, NULL);
  
  unity_webapps_npapi_add_array_support (pWrapper, get_integration_script_by_idx);
  
  // validate the "reserved" hidden description part
  pWrapper->reserved = ppDescription;
  
  // custom deallocator
  unity_webapps_npapi_set_deallocator (pWrapper, free_repo_descriptions);
  
  return pObject;
}


static NPVariant
integration_script_count_property_get (NPObject * object)
{
  NPAPI_LOG (INFO, "%s", __func__);
  
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = (UnityWebappsJavascriptObjectWrapper *) object;
  
  NPVariant result;
  INT32_TO_NPVARIANT (0, result);
  
  g_return_val_if_fail (NULL != pWrapper, result);
  g_return_val_if_fail (NULL != pWrapper->reserved, result);
  
  gsize count = 0;
  
  UnityWebappsIntegrationScriptDescription **
    ppDescription = pWrapper->reserved;
  
  while (ppDescription[count] != NULL) ++count;
  
  INT32_TO_NPVARIANT(count, result);
  
  return result;
}


static NPVariant
get_integration_scripts (NPP instance
			 , NPObject * object
			 , const NPVariant args[]
			 , uint32_t argcnt)
{
  NPAPI_LOG (INFO, "%s", __func__);
  
  NPVariant result;
  NULL_TO_NPVARIANT (result);
  
  UnityWebappsJavascriptObjectWrapper *
    pWrapper = (UnityWebappsJavascriptObjectWrapper *) object;
  
  g_return_val_if_fail (NULL != pWrapper, result);
  
  NPObject *
    repo_object = create_userscript_repo_result_object (pWrapper->instance);
  
  g_return_val_if_fail (NULL != repo_object, result);
  
  OBJECT_TO_NPVARIANT (repo_object, result);
  
  return result;
}


