// ==UserScript==
// @name           Unity GMail
// @namespace      UnityGMail
// @description    Unity GMail Integration
// @include        https://mail.google.com/*
// @version        0.9
// @author         WebApps Team
// @require        utils.js
// @require        google-common.js
// ==/UserScript==

var doc = null, pane = null;

function isCorrectPage() {
    var i, ids = ['canvas_frame', 'hist_frame', 'js_frame'];

    for (i = 0; i < ids.length; i++) {
        if (!document.getElementById(ids[i])) {
            return false;
        }
    }

    if (parent.document !== document) {
        return false;
    }

    return true;
}

function getNumber(str) {
    try {
        var onlyNumber = str.match(/\(([0-9]|,)+\)/)[0].match(/([0-9]|,)+/)[0];

        return onlyNumber.replace(/,/g, "");
    } catch (x) {
        return '0';
    }
}

function getLabels() {
    var i, res = [];

    var snapshot = doc.evaluate('//div[2]/div/div/div[5]/div/*/div/div/div[2]/span/a',
                                pane, null, XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null);

    for (i = 0; i < snapshot.snapshotLength; i++) {
        var node = snapshot.snapshotItem(i);

        res.push({ name: node.textContent.match(/^[^(]+/)[0],
                   count: getNumber(node.textContent),
                   link: node.href });
    }

    return res;
}

function doMatrixIntegration() {
    doMainMenuIntegration(doc);

    var i, labels = getLabels();
    for (i = 0; i < labels.length; i++) {
        Unity.addAction('/' + labels[i].name, makeRedirector(labels[i].link));
    }
}

function checkMessangesCount() {
    var inboxLink = doc.getElementsByClassName("nU n1")[0].getElementsByTagName("a")[0];
    var numMessages = getNumber(inboxLink.text);

    Unity.MessagingIndicator.showIndicator(_("Inbox"), { count: numMessages,
                                                         callback: makeRedirector(inboxLink.href) });

    var i, labels = getLabels();
    for (i = 0; i < labels.length; i++) {
        Unity.MessagingIndicator.showIndicator(labels[i].name, { count: labels[i].count,
                                                                 callback: makeRedirector(labels[i].link) });
    }
}


function unityLoaded() {
    function composeNewMessage() {
        var compose = doc.evaluate('//div[@role="navigation"]/div[1]/div[1]/div[1][@role="button"]', doc, null, XPathResult.ANY_UNORDERED_NODE_TYPE, null).singleNodeValue;

        click(compose);
    }

    Unity.MessagingIndicator.addAction(_("Compose New Message"),
                                       composeNewMessage);
    Unity.Launcher.addAction(_("Compose New Message"),
			     composeNewMessage);

    setInterval(checkMessangesCount, 2000);
    checkMessangesCount();

    doMatrixIntegration();
}

if (isCorrectPage()) {
    setTimeout(function wait() {
        doc = document.getElementById('canvas_frame').contentDocument;

        pane = doc.evaluate('//div[@role="navigation"]',
                            doc, null, XPathResult.ANY_UNORDERED_NODE_TYPE, null).singleNodeValue;

        if (!pane || !doc.getElementsByClassName("nU n1")) {
            setTimeout(wait, 1000);
            return;
        }
        Unity.init({ name: "GMail",
                     iconUrl: "http://mail.google.com/mail/images/2/mail_icon_32.png",
                     onInit: unityLoaded });
    }, 1000);
}
