// ==UserScript==
// @name          lastfm-unity-integration
// @include       http://last.fm/*
// @include       http://*.last.fm/*
// @include       https://last.fm/*
// @include       https://*.last.fm/*
// @version       0.9
// @author        WebApps Team
// ==/UserScript==

function isCorrectPage() {
    var i, ids = ['nowPlayingMeta', 'radioControlPlay', 'radioControlPause', 'radioControlSkip', 'webRadio'];

    for (i = 0; i < ids.length; i++) {
        if (!document.getElementById(ids[i])) {
            return false;
        }
    }

    return true;
}

function trim(str) {
    return str.replace(/^\s+|\s+$/g, '');
}

function getTrackInfo() {
    var title = document.title;
    var artLocation = null;
    var album = null;
    var artist = null;
    try {
        var meta = document.getElementById('nowPlayingMeta');
        artLocation = document.evaluate('div/h1/a/img', meta, null,
                                        XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue.src;

        title = document.title.match(/.+(?= - Last\.fm$)/)[0];

        artist = document.evaluate('div/h1/a[2]', meta, null,
                                   XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue.innerHTML;
        artist = trim(artist);

        if (title.substr(0, artist.length + 3) === artist + ' - ') {
            title = title.substr(artist.length + 3);

            var trackAlbum = document.getElementById('trackAlbum');
            album = document.evaluate('ul/li/div/strong/a', trackAlbum, null,
                                      XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue.textContent;
            trim(album);
        } else {
            artist = null;
        }
    } catch (x) {}

    if (!artLocation) {
        return null;
    }

    return {
        title: title,
        album: album,
        artist: artist,
        artLocation: artLocation
    };
}

function musicPlayerSetup() {
    Unity.MusicPlayer.init("last.fm");
    Unity.MusicPlayer.canGoPrevious = false;

    setTimeout(function retry() {
        var trackInfo = getTrackInfo();

        setTimeout(retry, 1500);

        if (trackInfo) {
            Unity.MusicPlayer.setTrack(trackInfo);
        }
    }, 1000);

    Unity.MusicPlayer.onPlayPause(function () {
        var paused = document.getElementById('webRadio').className === 'paused';
        var pause = document.getElementById('radioControlPause');
        var play = document.getElementById('radioControlPlay');
        if (paused) {
            play.click();
        } else {
            pause.click();
        }

        Unity.MusicPlayer.playbackState = !paused;
    });

    Unity.MusicPlayer.onNext(function () {
        document.getElementById('radioControlSkip').click();
    });
}

if (isCorrectPage()) {
    Unity.toDataURL('http://cdn.lst.fm/flatness/header/headersprite.1.png', function (aResult, uri) {
        Unity.init({ name: "last.fm",
                     iconUrl: uri,
                     onInit: musicPlayerSetup });
    }, 8, 164, 95, 31);
}

